import { __awaiter } from 'tslib';
import * as i0 from '@angular/core';
import { InjectionToken, Injectable, Inject, NgModule } from '@angular/core';
import { FpjsClient } from '@fingerprintjs/fingerprintjs-pro-spa';
export { CacheLocation, FingerprintJSPro, InMemoryCache, LocalStorageCache, SessionStorageCache, defaultEndpoint, defaultScriptUrlPattern, defaultTlsEndpoint } from '@fingerprintjs/fingerprintjs-pro-spa';

const FINGERPTINTJS_PRO_ANGULAR_SETTINGS_TOKEN = new InjectionToken('ng-fingerprintjspro-settings-token', {
    factory: () => ({ clientOptions: { loadOptions: { apiKey: '' } } }),
});

const packageVersion = '2.0.0';

/**
 * Inject FingerprintjsProAngularService and use it to make identification requests.
 *
 * @example ```typescript
 * import { Component, OnInit } from '@angular/core';
 * import { FingerprintjsProAngularService } from 'fingerprintjs-pro-angular';
 *
 * @Component({
 *   selector: 'app-home',
 *   templateUrl: './home.component.html',
 *   styleUrls: ['./home.component.css']
 * })
 * export class HomeComponent implements OnInit {
 *
 *   constructor(private fingerprintjsProAngularService: FingerprintjsProAngularService) { }
 * //^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 *   visitorId = 'Press "Identify" button to get visitorId';
 *
 *   async onIdentifyButtonClick() : Promise<void> {
 *     const data = await this.fingerprintjsProAngularService.getVisitorData();
 * //  ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 *     this.visitorId = data.visitorId;
 *     this.extendedResult = data;
 *   }
 * }
 * ```
 */
class FingerprintjsProAngularService {
    constructor(settings) {
        const { loadOptions } = settings.clientOptions;
        const clientOptions = Object.assign(Object.assign({}, settings.clientOptions), { loadOptions: Object.assign(Object.assign({}, loadOptions), { integrationInfo: [...(loadOptions.integrationInfo || []), `fingerprintjs-pro-angular/${packageVersion}`] }) });
        this.fingerprintJsClient = new FpjsClient(clientOptions);
        this.fingerprintJsClientInitPromise = this.fingerprintJsClient.init();
    }
    getVisitorData(options, ignoreCache) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fingerprintJsClientInitPromise;
            return this.fingerprintJsClient.getVisitorData(options, ignoreCache);
        });
    }
    clearCache() {
        return this.fingerprintJsClient.clearCache();
    }
}
FingerprintjsProAngularService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FingerprintjsProAngularService, deps: [{ token: FINGERPTINTJS_PRO_ANGULAR_SETTINGS_TOKEN }], target: i0.ɵɵFactoryTarget.Injectable });
FingerprintjsProAngularService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FingerprintjsProAngularService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FingerprintjsProAngularService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root',
                }]
        }], ctorParameters: function () {
        return [{ type: undefined, decorators: [{
                        type: Inject,
                        args: [FINGERPTINTJS_PRO_ANGULAR_SETTINGS_TOKEN]
                    }] }];
    } });

/**
 * Include FingerprintjsProAngularModule using `forRoot` method.
 *
 * @example ```typescript
 * import { NgModule } from '@angular/core';
 * import { FingerprintjsProAngularModule } from 'fingerprintjs-pro-angular';
 * // ...
 *
 * @NgModule({
 *   declarations: [AppComponent],
 *   imports: [
 *     BrowserModule,
 *     FingerprintjsProAngularModule.forRoot({loadOptions: {apiKey: 'your-fpjs-public-api-key'}})
 * //  ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 *   ],
 *   providers: [],
 *   bootstrap: [AppComponent]
 * })
 * export class AppModule { }
 * ```
 */
class FingerprintjsProAngularModule {
    static forRoot(clientOptions) {
        return {
            ngModule: FingerprintjsProAngularModule,
            providers: [
                {
                    provide: FINGERPTINTJS_PRO_ANGULAR_SETTINGS_TOKEN,
                    useValue: {
                        clientOptions,
                    },
                },
                FingerprintjsProAngularService,
            ],
        };
    }
}
FingerprintjsProAngularModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FingerprintjsProAngularModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
FingerprintjsProAngularModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "15.2.10", ngImport: i0, type: FingerprintjsProAngularModule });
FingerprintjsProAngularModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FingerprintjsProAngularModule });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.10", ngImport: i0, type: FingerprintjsProAngularModule, decorators: [{
            type: NgModule
        }] });

/*
 * Public API Surface of fingerprintjs-pro-angular
 */

/**
 * Generated bundle index. Do not edit.
 */

export { FingerprintjsProAngularModule, FingerprintjsProAngularService };
//# sourceMappingURL=fingerprintjs-fingerprintjs-pro-angular.mjs.map
